import java.awt.*;
import java.awt.AWTEvent;
import java.lang.*;
import java.awt.event.*;
import java.awt.swing.*;
import java.awt.Color;
import java.awt.Font;

/**
TicTacToe class implements 2 listeners, ActionListener and ItemListener.
It also inherits from JPanel class. TicTacToe has two modes, SIMPLE or FANCY,
In Simple mode, the game does not show some of the Swing features such as change
of Look and Feel, changing game options, floatable toolbars, etc. The purpose of
these two modes is to show that using SWING features can enhance your
GUI. 

*/

public class TicTacToe extends JPanel
                        implements ActionListener, ItemListener {

	static final byte NO_ONE  = 0;
    static final byte PLAYER1 = 1; 
    static final byte PLAYER2 = 4; 
    static final byte TIE = -1;  // no winner
    byte player = PLAYER1;    // current active player
    byte winner = NO_ONE;    
	static final byte RED = 1;
	static final byte BLUE = 3;
	static final byte LETTERO = 2;
	static final byte LETTERX = 4;
	static final byte SIMPLE = 0;
	static final byte FANCY = 1;
	byte firstcolor = RED;  //default start color
	byte firstletter = LETTERO; //default start color
	int lastmove=0; // 0 means game has not start yet.

	//tictactoe type

	byte type;

	// message label
	JLabel MsgLabel;

	byte CURRENT=1;
	byte NEXT=2;
	
	// Menu used in the panel
	JMenu file, options, help;

	// buttons used in the gameboard

	JButton b1, b2, b3, b4,b5,b6,b7,b8,b9;  
	byte board[] = new byte[10];  //board[0] is not used.

	/** load all the images used by the game */

	ImageIcon IconRedO = new ImageIcon("images\\redO.gif");
	ImageIcon IconRedX = new ImageIcon("images\\redX.gif");
	ImageIcon IconBlueX = new ImageIcon("images\\blueX.gif");
	ImageIcon IconBlueO = new ImageIcon("images\\blueO.gif");
	ImageIcon IconInvisible = new ImageIcon("images\\invisible.gif");
	ImageIcon HelpIcon = new ImageIcon("images\\help.gif");
	ImageIcon NewIcon = new ImageIcon("images\\newBig.gif");
	ImageIcon NewSmallIcon = new ImageIcon("images\\new.gif");
	ImageIcon QuitIcon = new ImageIcon("images\\a1d.gif");
	ImageIcon DukeWaveSmallIcon = new ImageIcon("images\\dukeWaveSmall.gif");
	ImageIcon DukeMagnifySmallIcon = new ImageIcon("images\\dukeMagnifySmall.gif");
	ImageIcon DukeWaveRedIcon = new ImageIcon("images\\dukeWaveRed.gif");
	ImageIcon Duke3Icon = new ImageIcon("images\\duke3.gif");

	/** default icons */

	ImageIcon firstPlayerIcon = IconRedO;
	ImageIcon secondPlayerIcon = IconBlueX;

	/** if TicTacToe is called without a parameter, assume SIMPLE mode.
	 */
	public TicTacToe() {
		new TicTacToe(FANCY);
	}

    public TicTacToe(byte inputtype) {
		super();
		setBorder(BorderFactory.createEtchedBorder());

		/** vertical layout for tictactoe */
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));   

		/** create toolbar */

		JToolBar toolbar;
		toolbar = createToolBar();
		
		JMenuBar menuBar;
		menuBar = createMenuBar();

		/** create game board */ 

		JPanel gameboard = new JPanel();
		gameboard.setLayout(new GridLayout(3, 3));

		/**		gameboard & board layout 
		 				 COL
		 			R   1  2  3
		 			O   4  5  6
		  			W   7  8  9
		 */

		/** row 1 */

		b1 = new JButton(IconInvisible);
		b1.setActionCommand("pos1");
		//b1.setBorderPainted(true);

		b2 = new JButton(IconInvisible);

		b2.setActionCommand("pos2");

		b3 = new JButton(IconInvisible);
		b3.setActionCommand("pos3");

		/** row 2*/
		b4 = new JButton(IconInvisible);
		b4.setActionCommand("pos4");

		b5 = new JButton(IconInvisible);
		b5.setActionCommand("pos5");

		b6 = new JButton(IconInvisible);
		b6.setActionCommand("pos6");

		/** row 3*/
		b7 = new JButton(IconInvisible);
		b7.setActionCommand("pos7");

		b8 = new JButton(IconInvisible);
		b8.setActionCommand("pos8");

		b9 = new JButton(IconInvisible);
		b9.setActionCommand("pos9");

		//Listen for actions 
		b1.addActionListener(this);
		b2.addActionListener(this);
		b3.addActionListener(this);
		b4.addActionListener(this);
		b5.addActionListener(this);
		b6.addActionListener(this);
		b7.addActionListener(this);
		b8.addActionListener(this);
		b9.addActionListener(this);


		//Add Components to this board panel. 
		gameboard.add(b1);
		gameboard.add(b2);
		gameboard.add(b3);
		gameboard.add(b4);
		gameboard.add(b5);
		gameboard.add(b6);
		gameboard.add(b7);
		gameboard.add(b8);
		gameboard.add(b9);

		/** set up msg panel */

		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new BorderLayout()); 

		MsgLabel = new JLabel("Click on the board to begin",Duke3Icon,JLabel.LEFT);
		Font font = new Font("serif", Font.BOLD, 14);
        MsgLabel.setFont(font);
		middlePanel.add(MsgLabel);

		/** add all the components together */
		add (menuBar);
		add (Box.createHorizontalGlue());
		add (toolbar);
		add (Box.createHorizontalGlue());
		add (gameboard); 
		add (Box.createHorizontalGlue());
		add (middlePanel); 

		/** clear the board array */
		InitBoard();
		
		if (inputtype == SIMPLE) {
			type = SIMPLE;
			toolbar.setFloatable(false);
		} else if (inputtype == FANCY) {
			type = FANCY;
			toolbar.setFloatable(true);
			setRollOverIcon();
		}
    }

	public void setRollOverIcon() { 
		ImageIcon roIcon;

		if (firstcolor == RED) {
			if (firstletter == LETTERO) {
				roIcon = IconRedO;
			} else {
				roIcon = IconRedX;
			}		
		}
		else {
			if (firstletter == LETTERO) {
				roIcon = IconBlueO;
			} else {
				roIcon = IconBlueX;
			}		
		}

		b1.setRolloverIcon(roIcon);
		b2.setRolloverIcon(roIcon);
		b3.setRolloverIcon(roIcon);
		b4.setRolloverIcon(roIcon);
		b5.setRolloverIcon(roIcon);
		b6.setRolloverIcon(roIcon);
		b7.setRolloverIcon(roIcon);
		b8.setRolloverIcon(roIcon);
		b9.setRolloverIcon(roIcon);

		b1.setRolloverEnabled(true);
		b2.setRolloverEnabled(true);
		b3.setRolloverEnabled(true);
		b4.setRolloverEnabled(true);
		b5.setRolloverEnabled(true);
		b6.setRolloverEnabled(true);
		b7.setRolloverEnabled(true);
		b8.setRolloverEnabled(true);
		b9.setRolloverEnabled(true);

	}

	public void setOptions(boolean set) {
		options.setEnabled(set);
	}

    public boolean moveRow(byte current_player) {
        // Search the rows for a near win
        for (int x=0; x<3; x++) {
            byte sum = 0;
            for (int y=1; y<4; y++) {
                sum += board[(x*3)+y];
			}
            // Almost a win -- block it
            if (sum == (current_player*2)) {
                for (int y=1; y<4; y++)  {
					int potmove = (x*3)+y; // potential move

                    if (board[potmove] == NO_ONE) {
                        board[potmove] = PLAYER2;
						addMoveToBoard(potmove, PLAYER2);
						lastmove = potmove;
                        return true;
					}
				}
			}
		}
        return false;
    }

    public boolean moveColumn(byte current_player) {
        // Search the columns for a near win
        for (int x=1; x<4; x++) {
            byte sum = 0;
            for (int y=0; y<3; y++)
                sum += board[x+(y*3)];
            // Almost a win -- block it
            if (sum == (current_player*2)) {
                for (int y=0; y<3; y++)	{
					int potmove = x+(y*3); // potential move
                    if (board[potmove] == NO_ONE) {
                        board[potmove] = PLAYER2;
						addMoveToBoard(potmove, PLAYER2);
						lastmove = potmove;
                        return true;
                    }
				}
			}
        }
        return false;
    }

    public boolean moveDiagonal(byte current_player) {
        // Search the diagonals for a near win
        int p1[] = {1, 5, 9};
        int p2[] = {3, 5, 7};
        int sum = 0;

        for (int i=0; i<3; i++) {
            sum += board[p1[i]];
		}
        if (sum == (current_player*2)) {
            for (int i=0; i<3; i++) {
                if (board[p1[i]] == NO_ONE) {
                    board[p1[i]] = PLAYER2;
					addMoveToBoard(p1[i], PLAYER2);
					lastmove = p1[i];
                    return true;
                }
			}
		}

        sum = 0;
        for (int i=0; i<3; i++)
            sum += board[p2[i]];
        if (sum == (current_player*2))
            for (int i=0; i<3; i++)
                if (board[p2[i]] == NO_ONE) {
                    board[p2[i]] = PLAYER2;
					addMoveToBoard(p2[i], PLAYER2);
					lastmove = p2[i];
                    return true;
                }
        return false;
    }

    // Computer AI
    // 1. Search for possible wins for me
    // 2. Search for possible wins for opponent
    // 3. Perform most strategic move
    public void computerMove() {
        // Can I win
        if (moveRow(PLAYER2)) {
//			System.out.println("PLAYER2 moveRow, lastmove=" + lastmove);
            return;
		}
        if (moveColumn(PLAYER2)) {
//			System.out.println("PLAYER2 moveColumn, lastmove=" + lastmove);
            return;
		}
	
        if (moveDiagonal(PLAYER2)) {
//			System.out.println("PLAYER2 moveColumn, lastmove=" + lastmove);
            return;
		}
            
        // Can he/she win
        if (moveRow(PLAYER1)) {
//			System.out.println("block PLAYER1 moveRow, lastmove=" + lastmove);
            return;
		}
        if (moveColumn(PLAYER1)) {
//			System.out.println("block PLAYER1 moveColumn, lastmove=" + lastmove);
            return;
		}
        if (moveDiagonal(PLAYER1)) {
//        	System.out.println("block PLAYER1 moveDiagonal, lastmove=" + lastmove);
		    return;
		}

        // Most stategic move
        if (board[5] == NO_ONE) {
            board[5] = PLAYER2;
			addMoveToBoard(5, PLAYER2);
			lastmove = 5;
            return;
        }

        // Very Special cases
        if ((board[5] == PLAYER2) && (board[9] == NO_ONE) &&
            (board[8] == PLAYER1) && (board[3] == PLAYER1)) {
            board[9] = PLAYER2;
			addMoveToBoard(9, PLAYER2);
			lastmove = 9;
            return;
        }

        if ((board[5] == PLAYER2) && (board[9] == NO_ONE) &&
            (board[7] == PLAYER1) && (board[6] == PLAYER1)) {
            board[9] = PLAYER2;
			addMoveToBoard(9, PLAYER2);
			lastmove = 9;
            return;
        }

        if ((board[5] == PLAYER2) && (board[7] == NO_ONE) &&
            (board[8] == PLAYER1) && (board[1] == PLAYER1)) {
            board[7] = PLAYER2;
			addMoveToBoard(7, PLAYER2);
			lastmove = 7;
            return;	
        }

        // Special cases
        int r[];
        if ((board[5] == PLAYER2) && 
            (((board[1] == PLAYER1) && (board[9] == PLAYER1)) ||
            ((board[3] == PLAYER1) && (board[7] == PLAYER1)))) {
            int rtmp1[] = {5, 2, 4, 6, 8, 1, 3, 7, 9};
            r = rtmp1;
        }
        else if ((board[5] == PLAYER2) &&
                (board[2] == PLAYER1) && (board[4] == PLAYER1)) {
            int rtmp2[] = {5, 1, 4, 6, 8, 2, 3, 7, 9};
            r = rtmp2;
        }
        else if ((board[5] == PLAYER2) &&
                (board[2] == PLAYER1) && (board[6] == PLAYER1)) {
            int rtmp3[] = {5, 3, 4, 6, 8, 1, 2, 7, 9};
            r = rtmp3;
        } 
        else if ((board[5] == PLAYER2) &&
                (board[8] == PLAYER1) && (board[4] == PLAYER1)) {
            int rtmp4[] = {5, 7, 4, 6, 8, 1, 3, 2, 9};
            r = rtmp4;
        }
        else if ((board[5] == PLAYER2) &&
                (board[8] == PLAYER1) && (board[6] == PLAYER1)) {
            int rtmp5[] = {5, 9, 4, 6, 8, 1, 3, 7, 2};
            r = rtmp5;
        }
        else {	
            int rtmp6[] = {5, 1, 3, 7, 9, 2, 4, 6, 8};
            r = rtmp6;
        }

        for (int i=1; i<9; i++)
            if (board[r[i]] == NO_ONE) {
                board[r[i]] = PLAYER2;
				addMoveToBoard(r[i], PLAYER2);
				lastmove = r[i];
                return;
            }
    }

    public void checkwinner() {
        // Check for diagonal win
        if (((board[3] + board[5] + board[7]) == 3) ||
            ((board[1] + board[5] + board[9]) == 3)) {
            winner = PLAYER1;
            return;
        }
        else if (((board[3] + board[5] + board[7]) == 12) ||
                ((board[1] + board[5] + board[9]) == 12)) {
            winner = PLAYER2;
            return;
        }

        // Search the columns for a win
        for (int x=1; x<4; x++) {
            byte sum = 0;
            for (int y=0; y<3; y++)
                sum += board[x+(y*3)];
            if (sum == 3) {
                winner = PLAYER1;
                return;
            }
            else if (sum == 12) {
                winner = PLAYER2;
                return;
            }
        }

        // Search the rows for a near win
        for (int x=0; x<3; x++) {
            byte sum = 0;
            for (int y=1; y<4; y++)
                sum += board[(x*3)+y];
            if (sum == 3) {
                winner = PLAYER1;
                return;
            }
            else if (sum == 12) {
                winner = PLAYER2;
                return;
            }
        }

        // Check for all squares filled
        for (int i=1; i<10; i++)
            if (board[i] == NO_ONE)
                return;
        winner = TIE;
        return;
    }

	/** keeps the corresponding value of each button */

    public void InitBoard() {
        for (int i=1; i<10; i++)
            board[i] = NO_ONE;
	}

	/** reinitialize the game board */

	public void ResetBoard() {
		b1.setIcon(IconInvisible);
		b2.setIcon(IconInvisible);
		b3.setIcon(IconInvisible);
		b4.setIcon(IconInvisible);
		b5.setIcon(IconInvisible);
		b6.setIcon(IconInvisible);
		b7.setIcon(IconInvisible);
		b8.setIcon(IconInvisible);
		b9.setIcon(IconInvisible);
		InitBoard();
		winner = NO_ONE;
		lastmove = 0;     
		//enable changing color,letter,computer go first menus
		if (type == FANCY) {
			options.setEnabled(true);
			setRollOverIcon(); 
		} 

		MsgLabel.setText("Click on the Board to begin again");
		player=PLAYER1;
	}

	public void setPlayerIcon() {
		if ((firstcolor == RED) && (firstletter == LETTERO)) {
			firstPlayerIcon = IconRedO;
			secondPlayerIcon = IconBlueX;
		} else if ((firstcolor == RED) && (firstletter == LETTERX)) {
			firstPlayerIcon = IconRedX;
			secondPlayerIcon = IconBlueO;
		} else if ((firstcolor == BLUE) && (firstletter == LETTERO)) {
			firstPlayerIcon = IconBlueO;
			secondPlayerIcon = IconRedX;
		} else if ((firstcolor == BLUE) && (firstletter == LETTERX)) {
			firstPlayerIcon = IconBlueX;
			secondPlayerIcon = IconRedO;
		}
	}

	String turnlabel(byte turn) {

		String label;

		if (turn == CURRENT) {
			if (firstcolor == RED) {
				label = new String("Red ");
			}
			else {
				label = new String("Blue ");
			}
			if (firstletter == LETTERO) {
				label = label + 'O';
			} else {
				label = label + 'X';
			}
			return label;
		} else {
			if (firstcolor == RED) {
				label = new String("Blue ");
			}
			else {
				label = new String("Red ");
			}
			if (firstletter == LETTERO) {
				label = label + 'X';
			} else {
				label = label + 'O';
			}
			return label;
		}
	}


	public void itemStateChanged(ItemEvent e) {
	   AbstractButton ab = (AbstractButton) e.getSource();
	   String itemText = ab.getText();

//	   System.out.println("itemstatechanged = " + itemText);

        if (ab.isSelected()) {
            if (itemText.equals("Red")) {
				firstcolor = RED;
				setPlayerIcon();
				ResetBoard();
                System.out.println("red selected in choosecolorchanger");

            } else if (itemText.equals("Blue")) {
				firstcolor = BLUE;
				setPlayerIcon();
				ResetBoard();
                System.out.println("blue selected in choosecolorchanger");

            } else if (itemText.equals("X")) {
				firstletter=LETTERX;
				setPlayerIcon();
				ResetBoard();
				System.out.println("X selected in chooseletterchanger");

			} else if (itemText.equals("O")) {
				firstletter=LETTERO;
				setPlayerIcon();
				ResetBoard();
			//	System.out.println("O selected in chooseletterchanger");
			} else if (itemText.equals("Computer go first")) {
				System.out.println("computer go first selected in menu");
				
				ResetBoard();
				MsgLabel.setText(turnlabel(NEXT));
				player = PLAYER2;
				computerMove();

				player = PLAYER1;
				MsgLabel.setText(turnlabel(CURRENT));

				// disable options menu until someone won
				options.setEnabled(false);
			}
        }
	}

    public void actionPerformed(java.awt.event.ActionEvent e) {

		String command;
		int move=0;

		command = e.getActionCommand();

		if (command.equals("New")) {
			ResetBoard();
		//	System.out.println("New game");
			return;

		} else if (command.equals("Quit")) {
			System.out.println("Quit");
			System.exit(0);
			return;
		} else if (command.equals("Mac")) { //not used right now.
			try {				
				UIManager.setLookAndFeel("java.awt.swing.plaf.mac.MacLookAndFeel");
				SwingUtilities.updateComponentTreeUI(this);
			} catch (UnsupportedLookAndFeelException exc) {
				System.err.println("Unsupported LookAndFeel: Mac");
			} catch (IllegalAccessException iexc) {
				System.err.println("IllegalAccessException LookAndFeel: Mac");
			} catch (InstantiationException inexc) {
				System.err.println("InstanticationException LookAndFeel: Mac");
			} catch (ClassNotFoundException cexc) {
				System.err.println("ClassNotFoundException LookAndFeel: Mac");
			}

			return;
		} else if (command.equals("Cross")) {
			try {				
				UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
				SwingUtilities.updateComponentTreeUI(this);
			} catch (UnsupportedLookAndFeelException exc) {
				System.err.println("Unsupported LookAndFeel: Metal");
			} catch (IllegalAccessException iexc) {
				System.err.println("IllegalAccessException LookAndFeel: Metal");
			} catch (InstantiationException inexc) {
				System.err.println("InstanticationException LookAndFeel: Metal");
			} catch (ClassNotFoundException cexc) {
				System.err.println("ClassNotFoundException LookAndFeel: Metal");
			}

			return;
		} else if (command.equals("Motif")) {
			
			try {				
				UIManager.setLookAndFeel("java.awt.swing.plaf.motif.MotifLookAndFeel");
				SwingUtilities.updateComponentTreeUI(this);
			} catch (UnsupportedLookAndFeelException exc) {
				System.err.println("Unsupported LookAndFeel: Motif");
			} catch (IllegalAccessException iexc) {
				System.err.println("IllegalAccessException LookAndFeel: Motif");
			} catch (InstantiationException inexc) {
				System.err.println("InstanticationException LookAndFeel: Motif");
			} catch (ClassNotFoundException cexc) {
				System.err.println("ClassNotFoundException LookAndFeel: Motif");
			}

			return;
		} else if (command.equals("Windows")) {
			String syslandf = UIManager.getSystemLookAndFeelClassName();

			try {
				UIManager.setLookAndFeel("java.awt.swing.plaf.windows.WindowsLookAndFeel");
				SwingUtilities.updateComponentTreeUI(this);
			} catch (UnsupportedLookAndFeelException exc) {
				System.err.println("Unsupported LookAndFeel: Windows");
			} catch (IllegalAccessException iexc) {
				System.err.println("IllegalAccessException LookAndFeel: Windows");
			} catch (InstantiationException inexc) {
				System.err.println("InstanticationException LookAndFeel: Windows");
			} catch (ClassNotFoundException cexc) {
				System.err.println("ClassNotFoundException LookAndFeel: Windows");
			}
			return;
		} 


		/** react to button events */
		/** resetboard from previous game **/

		if ((winner != NO_ONE) || (winner == TIE)) {
            ResetBoard();
        }
		if (player !=PLAYER1)
			return;

		/** get move */
		//	System.out.println("charAt3= " + command.substring(3)); 
		
		try {
			move = Integer.parseInt(command.substring(3));
		} catch (NumberFormatException n) {
			System.out.println("NumberFormatException");
		}

		//  System.out.println("move=" + move);
		
		/** check for valid moves */
		if ((move >= 1) && (move <= 9) && (board[move] == NO_ONE)) {
			board[move] = player;
			if (lastmove == 0) {   // 1st move
				options.setEnabled(false);
			}
			lastmove = move;
			addMoveToBoard(move, PLAYER1);
			debugBoard(); /** */
			checkwinner();

			/** computer's move */

			if (winner == TIE) { //tie game
				MsgLabel.setText("TIE Game.");
				if (type == FANCY) {
					options.setEnabled(true);
				}
				return;  		
			}if (winner !=NO_ONE) {
				MsgLabel.setText(turnlabel(CURRENT) + " WON!");
				if (type == FANCY) {
					options.setEnabled(true);
				}
				return; // found a winner
			}

			MsgLabel.setText(turnlabel(NEXT));
			player = PLAYER2;
			computerMove(); // computer move recommend move & adds it to the board
		//	System.out.println("move recommended by computerMove=" + lastmove);

		/** */	debugBoard(); 
			checkwinner();
			if (winner == TIE) { //tie game
				MsgLabel.setText("TIE Game.");
				if (type == FANCY) {
					options.setEnabled(true);
				}
				return;  		
			}if (winner !=NO_ONE) {
				MsgLabel.setText(turnlabel(NEXT) + " WON!");
				if (type == FANCY) {
					options.setEnabled(true);
				}
				return; // found a winner
			}

			MsgLabel.setText(turnlabel(CURRENT) + "'s turn");
			player = PLAYER1;
			return;
		} else return;
    }
    
	/** add a move to the game board 
	 */

	private void addMoveToBoard(int move, byte player) {

		ImageIcon drawIcon=null;

		if (player == PLAYER1) {
			drawIcon = firstPlayerIcon;

			// System.out.println("about to draw firstplayer icon");
		} else {
			drawIcon = secondPlayerIcon;
			// System.out.println("about to draw secondplayer icon");
		}

		// System.out.println("drawing pos" + move);
		switch (move) {
		case 1:
			b1.setIcon(drawIcon);
			if (type == FANCY)
				b1.setRolloverIcon(drawIcon);
			break;
		case 2:
			b2.setIcon(drawIcon);
			if (type == FANCY)
				b2.setRolloverIcon(drawIcon);
			break;
		case 3:
			b3.setIcon(drawIcon);
			if (type == FANCY)
				b3.setRolloverIcon(drawIcon);
			break;
		case 4:
			b4.setIcon(drawIcon);
			if (type == FANCY)
				b4.setRolloverIcon(drawIcon);
			break;
		case 5:
			b5.setIcon(drawIcon);
			if (type == FANCY)
				b5.setRolloverIcon(drawIcon);
			break;
		case 6:
			b6.setIcon(drawIcon);
			if (type == FANCY)
				b6.setRolloverIcon(drawIcon);
			break;
		case 7:
			b7.setIcon(drawIcon);
			if (type == FANCY)
				b7.setRolloverIcon(drawIcon);
			break;
		case 8:
			b8.setIcon(drawIcon);
			if (type == FANCY)
				b8.setRolloverIcon(drawIcon);
			break;
		case 9:
			b9.setIcon(drawIcon);
			if (type == FANCY)
				b9.setRolloverIcon(drawIcon);
			break;
		default:
			break;
		}
	}

	public void debugBoard() {
        System.out.println(board[1] + " " + board[2] + " " + board[3]);
        System.out.println(board[4] + " " + board[5] + " " + board[6]);
        System.out.println(board[7] + " " + board[8] + " " + board[9]);
        System.out.println("");
    }

	/**Toolbar used in the inner panel of TicTacToe game.
	*/
	private JToolBar createToolBar() {
		JToolBar toolbar = new JToolBar();
		JButton jb;

		/** create a button and add an event handler to the button.
			Add the button to the toolbar.
		 */

		toolbar.add (jb = new JButton ("New Game", NewIcon));
		jb.setToolTipText ("New Game");
		jb.setActionCommand("New");
		jb.addActionListener(this);
		toolbar.addSeparator();

		toolbar.add(jb=new JButton("Quit", DukeWaveSmallIcon));
		jb.setToolTipText ("Quit");
		jb.setActionCommand("Quit");
		jb.setHorizontalAlignment(JButton.RIGHT);
		jb.addActionListener(this);

		return toolbar;
	}


	/** create the menu bar inside the TicTacToe panel */

	private JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenuItem item;

		/** create file menu */

		file = new JMenu ("Game");
		file.setMnemonic ('G');
		file.add (item = new JMenuItem ("New Game",NewSmallIcon));
		item.setMnemonic ('N');
		item.setActionCommand("New");
		item.addActionListener(this);

		file.add (item = new JMenuItem ("Quit",DukeWaveSmallIcon));
		item.setMnemonic ('Q');
		item.setActionCommand("Quit");
		item.addActionListener(this);

		/** create options menu */
		options = new JMenu("Options");
		options.setMnemonic('O');

		JMenu colormenu = new JMenu("Change Color");
		
		ButtonGroup colorButtonGroup = new ButtonGroup();
		JRadioButtonMenuItem redButton = new JRadioButtonMenuItem("Red");
		JRadioButtonMenuItem blueButton = new JRadioButtonMenuItem("Blue");

		redButton.setSelected(true);
		redButton.setMnemonic('R');
		redButton.addItemListener(this);

		blueButton.setMnemonic('B');
		blueButton.addItemListener(this);
	
		colorButtonGroup.add(redButton);
		colorButtonGroup.add(blueButton);
		colormenu.add(redButton);
		colormenu.add(blueButton);
		options.add(colormenu);

		JMenu lettermenu = new JMenu("Change Letter");
		ButtonGroup letterButtonGroup = new ButtonGroup();
		JRadioButtonMenuItem OButton = new JRadioButtonMenuItem("O");
		JRadioButtonMenuItem XButton = new JRadioButtonMenuItem("X");
		letterButtonGroup.add(OButton);
		letterButtonGroup.add(XButton);

		OButton.setMnemonic('O');
		OButton.addItemListener(this);

		XButton.setMnemonic('X');
		XButton.addItemListener(this);
		
		/** add O, X buttons */
		options.addSeparator();
		lettermenu.add(OButton);
		lettermenu.add(XButton);
		options.add(lettermenu);

		/** create compute go first checkbox */

		JCheckBoxMenuItem compGoFirstCheckBox = new JCheckBoxMenuItem("Computer go first");

		compGoFirstCheckBox.setMnemonic('C');
		compGoFirstCheckBox.addItemListener(this);
		
		options.addSeparator();
		options.add(compGoFirstCheckBox);
	
		/** create Help menu */

		help = new JMenu ("Help");
		help.setMnemonic('H');
		help.add(item = new JMenuItem("Instruction", HelpIcon));
		item.setMnemonic('I');
		item.addActionListener (new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				doInstruction();
			}
		});

		help.add(item = new JMenuItem("About TicTacToe",DukeMagnifySmallIcon));
		item.setMnemonic('A');
		item.addActionListener (new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				doAboutTicTacToe();
			}
		});
		
		menuBar.add(file);
		menuBar.add(options);
		menuBar.add(help);
				
		return menuBar;
	}


	/** Create and popop up a JDialog with instruction for the TicTacToe game.

	*/

	void doInstruction() {
		// Create JDialog named 'dialog'
		final JDialog dialog = new JDialog ();

		dialog.addWindowListener(new WindowAdapter() {
		  public void windowClosing(WindowEvent e) {dialog.dispose();}
		});
		
    //	JTextArea  textarea  = new JTextArea ("Instruction For playing TicTacToe: To start TicTacToe, simpley click on the gameboard.");
	//	textarea.setEditable(false);
	//	textarea.setLineWrap(true);

		JLabel label1= new JLabel ("Instruction for playing TicTacToe:", JLabel.LEFT);
		JLabel label2= new JLabel ("simpley click on the gameboard.", JLabel.LEFT);

	//	dialog.getContentPane().add (textarea, BorderLayout.CENTER);
		dialog.getContentPane().add (label1, BorderLayout.NORTH);
		dialog.getContentPane().add (label2, BorderLayout.CENTER);

		JButton butt = new JButton ("Close Instruction");
		dialog.getContentPane().add (butt, BorderLayout.SOUTH);
		butt.addActionListener (new ActionListener () {
		  public void actionPerformed(ActionEvent e) {
			dialog.setVisible(false);
			dialog.dispose();
		  }
		});
		dialog.setSize (300, 300);
		dialog.setVisible(true);
		
	}
	void doAboutTicTacToe() {
		// Create JDialog named 'dialog'
		final JDialog dialog = new JDialog ();

		dialog.addWindowListener(new WindowAdapter() {
		  public void windowClosing(WindowEvent e) {dialog.dispose();}
		});
		JLabel  lab  = new JLabel ("TicTacToe version 1.0", JLabel.CENTER);
		dialog.getContentPane().add (lab, BorderLayout.CENTER);
		JButton butt = new JButton ("Close");
		dialog.getContentPane().add (butt, BorderLayout.SOUTH);
		butt.addActionListener (new ActionListener () {
		  public void actionPerformed(ActionEvent e) {
			dialog.setVisible(false);
			dialog.dispose();
		  }
		});
		dialog.setSize (200, 200);
		dialog.setVisible(true);
	}

	public static void main(String[] args) {
	/*
	 * Create a window.  Use JFrame since this window will include 
	 * lightweight components.
	 */
	JFrame frame = new JFrame("TicTacToe");	


	// Force SwingSet to come up in the Cross Platform L&F
	try {
//	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    // If you want the System L&F instead, comment out the above line and
	    // uncomment the following:
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception exc) {
	    System.err.println("Error loading L&F: " + exc);
	}


	/**defalut exit for Window close listener */

	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	};
	frame.addWindowListener(l);

	TicTacToe game = new TicTacToe();

	frame.getContentPane().add("Center", game);
 
	JMenuBar menuBar;
	menuBar = game.createMenuBar();
	frame.setJMenuBar(menuBar);

	JToolBar toolbar;
	toolbar = game.createToolBar();
	frame.getContentPane().add("North", toolbar);

	frame.pack();
	frame.show();
    }
}

